<?php
 
/**
 * Admin Ajax Class 
 * includes/admin/Ajax.php
 * @since 1.1.0
 */

# Namespace
namespace EasyVideoReviews\Admin;
 

# Exit if accessed directly
defined('ABSPATH') or die('Direct Script not Allowed');

# Main class 

class Ajax extends \EasyVideoReviews\Controller
{ 

    # Register hooks
    function register_hooks()
    {

        $actions = [
            'update_access_token' => [$this, 'update_access_token'],
            'update_accessibility' => [$this, 'update_accessibility'],
            'get_settings' => [$this, 'get_settings'],
            'update_settings' => [$this, 'update_settings'], 
            'reset_settings' => [$this, 'reset_settings'], 
            'hide_notices' => [$this, 'hide_notices'],
            'get_pages' => [$this, 'get_pages'],
        ];

        foreach ($actions as $action => $callback) {
            add_action('wp_ajax_' . $this->prefix . $action, $callback);

            // is test mode enabled?
            if ( $this->is_debug() ) {
                add_action('wp_ajax_nopriv_' . $this->prefix . $action, $callback);
            }
        }
    }

    /**
     * Check nonce
     */
    function check_nonce($nonce = '')
    {        
        if (!$nonce) {
            $nonce = $this->inputs('nonce');
        }

        if (!wp_verify_nonce($nonce, 'evr_admin')) {
            wp_send_json_error('Invalid nonce');
            wp_die();
        }
    }

    /**
     * Update access token for initial setup 
     */
    function update_access_token()
    { 
        $access_token = $this->input->access_token; # sanitized access token

        $this->update_option('access_token', $access_token);
    
        $this->send_json([ 'access_token' => $access_token, 'server' => $this->server() ], true, __('Access token updated', 'easy-video-reviews')); 
    }


    /**
     * Update accessibility
     */
    function update_accessibility()
    { 
        $is_pro = boolval($this->input->is_pro ?? false); # sanitized is pro

        $this->update_option('is_pro', $is_pro);

        $this->send_json([ 'is_pro', $is_pro ], true, __('Accessibility updated', 'easy-video-reviews'));
    } 

    /**
     * Get user settings 
     */
    function get_settings()
    {
        $options = $this->get_options(); 

        $this->send_json($options, true, __('Settings fetched', 'easy-video-reviews'));
    }

    /**
     * Update user settings  
     */
    function update_settings()
    {

        $inputs = (array) $this->input; # sanitized inputs
        $default_options = $this->app->options(); 

        foreach ($default_options as $key => $default) {

            if (isset($inputs[$key])) {
                $new_value = $inputs[$key];
                $this->update_option($key, $new_value);
            } 

        }
 
        $this->send_json($this->get_options(), true, __('Settings updated', 'easy-video-reviews'));
    }

    /**
     * Reset user settings   
     */
    function reset_settings()
    {
        $default_options = $this->app->options(); 

        foreach ($default_options as $key => $default) {
            $this->update_option($key, $default);
        }

        $this->send_json($this->get_options(), true, __('Settings reset', 'easy-video-reviews'));
 
    }
    
    /**
     * Get pages 
     */
    function get_pages()
    {
        $pages = get_posts([
            'post_type' => 'page',
            'post_status' => 'publish',
            'posts_per_page' => -1,
            'orderby' => 'title',
            'order' => 'ASC', 
            'fields' => ['ID', 'post_title', 'post_name'],
        ]);

        $pages = array_map(function($page) {
            return [
                'value' => $page->ID,
                'label' => $page->post_title ?? __('Untitled', 'easy-video-reviews'),
                'slug' => $page->post_name,
            ];
        }, $pages);

        $this->send_json($pages, true, __('Pages fetched', 'easy-video-reviews'));
    }

    /**
     * Hide notices
     */
    function hide_notices()
    {
        $id = $this->input->id; # sanitized id
        $transient = $this->input->transient; # sanitized transient

        $key = "show_{$id}_notice";

        if($transient == 'hide') {
            // hide key 
            $this->update_option($key, 'hide');
        } else {
            // set transient value 'hide' for transient * days in seconds
            $this->set_transient($key, 'hide', $transient * DAY_IN_SECONDS);
        }

        $this->send_json([
            'key' => $key,
            'transient' => $transient, 
        ]);
    }
}

/**
 * Instantiate the class when Ajax is called 
 */


(new Ajax())->register_hooks();