<?php

/**
 * Class Install
 * includes/classes/Install.php
 * Handles plugin installation
 * @since 1.3.0
 */

# Namespace
namespace EasyVideoReviews;

# Exit if accessed directly
defined('ABSPATH') or die('Direct Script not Allowed');

/**
 * Class Adjustment
 */

class Install extends \EasyVideoReviews\Controller
{
    # Register hooks
    public function init()
    {
        # Plugin activation
        register_activation_hook(EasyVideoReviews, [$this, 'activated']);

        # Plugin deactivation
        register_deactivation_hook(EasyVideoReviews, [$this, 'deactivated']);

        // redirect to admin page on activation this plugin
        add_action('admin_init', [$this, 'redirect_to_admin_page']);

        // plugin action links 
        add_action("plugin_action_links_" . plugin_basename(EasyVideoReviews), [$this, 'plugin_action_links']);

        // author uri
        add_filter('plugin_row_meta', [$this, 'plugin_row_meta'], 10, 2);
    }

    /**
     * activated
     */
    function activated()
    {
        /**
         * Init options
         */
        $this->init_options();

        /**
         * Init tables
         */
        $this->init_tables();

        /**
         * Hide admin notices
         */
        $this->init_admin_notices();

        /**
         * Init recording page
         */
        $this->init_template_pages();

        /**
         * Adjust token
         */
        $this->adjust_token();
    }

    /**
     * deactivated
     */
    function deactivated()
    {
        /**
         * Remove redirection
         */
        $this->delete_option('redirected');
    }

    /**
     * Init options
     */
    function init_options()
    {
        $default_options = $this->app->options();

        foreach ($default_options as $key => $value) {
            add_option($this->prefix . $key, $value);
        }

        // options will be automatically sycned if any of these options are updated
    }

    /**
     * Init tables
     */
    function init_tables()
    {
        // forms table
    }

    /**
     * Hide admin notices
     */
    function init_admin_notices()
    {
        $show_review_notice = $this->get_transient('show_review_notice', false) && $this->get_option('show_review_notice', true); 

        // show after 7 days
        if (!$show_review_notice) {
            // set hide after 7 days
            $this->set_transient('show_review_notice', 'hide', 7 * DAY_IN_SECONDS);
        }

        $show_affiliate_notice = $this->get_transient('show_affiliate_notice', false) && $this->get_option('show_affiliate_notice', true);

        // show after 14 days
        if (!$show_affiliate_notice) {
            // set hide after 14 days
            $this->set_transient('show_affiliate_notice', 'hide', 14 * DAY_IN_SECONDS);
        }
    }

    /**
     * Init recording page 
     */
    function init_template_pages()
    {

        // check if recording page is already created
        $recording_page_id = $this->get_option('recording_page_id', false);

        if (!$recording_page_id && $this->is_wc_installed) {
            // create recording page
            $recording_page_id = wp_insert_post([
                'post_title' => __('Leave a Review', 'easy-video-reviews'),
                'post_content' => '[evr-button]Leave a Review[/evr-button]',
                'post_status' => 'publish',
                'post_type' => 'page',
            ]);

            // update recording page id
            $this->update_option('recording_page_id', $recording_page_id);
        }

        // check if review page id is already created

        $review_page_id = $this->get_option('review_page_id', false);

        if (!$review_page_id) {
            // create review page
            $review_page_id = wp_insert_post([
                'post_title' => __('Video Review', 'easy-video-reviews'),
                'post_content' => 'This will NOT be displayed. This is just a placeholder page.',
                'post_status' => 'publish',
                'post_type' => 'page',
            ]);

            // update review page id
            $this->update_option('review_page_id', $review_page_id);
        }
    }

    /**
     * Redirect to admin page
     */
    function redirect_to_admin_page()
    {
        $redirected_after_activation = $this->get_option('redirected', false);

        if (!$redirected_after_activation) {
            // update option
            $this->update_option('redirected', 1);

            // redirect to admin page
            wp_redirect(admin_url('admin.php?page=easy-video-reviews'));
            exit;
        }
    }

    /**
     * Plugin action links
     */
    function plugin_action_links($links)
    {

        if ($this->has_access_token()) {
            $reviews_link = '<a href="' . esc_url(add_query_arg('page', 'easy-video-reviews', get_admin_url() . 'admin.php')) . '">' . __('Reviews', 'easy-video-reviews') . '</a>';
            $setting_link = '<a href="' . esc_url(add_query_arg('page', 'easy-video-reviews-settings', get_admin_url() . 'admin.php')) . '">' . __('Settings', 'easy-video-reviews') . '</a>';

            array_unshift($links, wp_kses_post($setting_link));
            array_unshift($links, wp_kses_post($reviews_link));
        }

        return $links;
    }

    /**
     * Plugin row meta
     */
    function plugin_row_meta($links, $file)
    {
        if ($file == plugin_basename(EasyVideoReviews)) {
            $row_meta = [
                'docs' => '<a href="' . esc_url('https://wppool.dev/docs-category/how-to-use-easy-video-reviews/') . '" aria-label="' . esc_attr__('View Easy Video Reviews documentation', 'easy-video-reviews') . '">' . esc_html__('Docs', 'easy-video-reviews') . '</a>',
                'support' => '<a href="' . esc_url('https://wordpress.org/support/plugin/easy-video-reviews/') . '" aria-label="' . esc_attr__('Visit Easy Video Reviews support forum', 'easy-video-reviews') . '">' . esc_html__('Support', 'easy-video-reviews') . '</a>',
            ];

            if ($this->has_access_token() && !$this->is_pro()) {
                $row_meta['upgrade'] = '<a style="color: #e74c3c; font-weight: 500" href="' . esc_url(admin_url('admin.php?page=easy-video-reviews-billing#plans')) . '">' . __('Upgrade', 'easy-video-reviews') . '</a>';
            }

            return array_merge($links, $row_meta);
        }

        return (array) $links;
    }

    /**
     * adjust_token
     */
    function adjust_token()
    {
        $new_token = $this->get_option('access_token', false);

        if( $new_token ) return false;

        $old_token = $this->get_option('evr_token', false);

        if( $old_token ) {
            $this->update_option('access_token', $old_token);
            $this->delete_option('evr_token');
        } 

    }
}

# Instance
(new Install())->init();
