<?php

/**
 * Abstract class for all Easy Video Reviews classes 
 * Base Controller
 * @since 1.3.8
 */

# Namespace
namespace EasyVideoReviews;

# Exit if accessed directly
defined('ABSPATH') or die('Direct Script not Allowed');

# Main class 
abstract class Controller
{

    /**
     * Helper trait
     */
    use \EasyVideoReviews\Helper;

    # Prefix
    protected $prefix = 'evr_';

    # App instance
    protected $app = null;
    # Inputs
    protected $input = null;

    # constructor
    function __construct()
    {
        $this->app = new App();
        $this->input = $this->get_inputs();
    }

    # render template 
    function view($template = '', $args = null)
    {
        if (!$template) return false;

        $template_file_location = EVR_PATH . 'templates/' . $template . '.php';

        if (file_exists($template_file_location)) {

            if ($args && is_array($args)) {
                extract($args);
            }

            include $template_file_location;
        }
    }

    # render HTML
    function render($html = '', array $vars = [])
    {
        if (empty($vars)) return $html;

        foreach ($vars as $key => $value) {
            $html = str_replace("{{" . $key . "}}", $value, $html);
        }
        return $html;
    }

    # Get inputs
    function get_inputs($param = null, $default = null)
    {
        try {
            $json = file_get_contents('php://input');
            // array 
            $request = json_decode($json);
        } catch (\Exception $e) {
        }

        if (!$request || empty($request)) {
            $request =  $_REQUEST;
        }

        if ($param) {
            return $request[$param] ?? $default ?? null;
        }

        return (object) $this->sanitize($request);
    }

    # Return JSON
    function send_json($data = [], $success = true, $message = null)
    {
        $payload = [
            'success' => boolval($success),
            'message' => '',
            'data' => '',
        ];

        if (is_array($data) || is_object($data)) {
            $payload['data'] = $data;
        } elseif ($data !== null) {
            $payload['message'] = $data;
        }

        if ($message) {
            $payload['message'] = $message;
        }

        wp_send_json($payload);
        wp_die();
    }

    // sanitize inputs
    function sanitize($data = [])
    {
        if (is_string($data)) {
            $data = sanitize_text_field($data);
        } elseif (is_array($data)) {
            foreach ($data as $key => &$value) {
                if (is_array($value)) {
                    $value = $this->sanitize($value);
                } else {
                    $value = sanitize_text_field($value);
                }
            }
        }

        return $data;
    }

    # Get access token
    function get_access_token()
    {
        $access_token = $this->get_option('access_token');

        return apply_filters('evr_access_token', $access_token);
    }


    # has access token
    function has_access_token()
    {
        return !empty($this->get_access_token());
    }

    # is pro access
    function is_pro()
    {
        return intval($this->get_option('is_pro'));
    }

    # is logged in 
    function is_logged_in()
    {
        // check if has access token saved in the website
        if (!$this->has_access_token()) return false;

        $server = new Client();

        $validate_user = $server->get('verify');

        return $validate_user;
    }

    # get server
    function server()
    {
        return apply_filters('evr_server', 'https://evr.wppool.dev/');
    }

    # host 
    function host()
    {
        return apply_filters('evr_host', $_SERVER['HTTP_HOST']);
    }

    # get options
    function get_options()
    {
        $default_options = $this->app->options();

        $options = [];

        foreach ($default_options as $key => $default) {
            $options[$key] = $this->get_option($key, $default);
        }

        return $options;
    }


    # get option
    function get_option($option = '', $default = null)
    { 
        $value = get_option($this->prefix . $option, $default); 

        if ($value !== null) {
            return maybe_unserialize($value);
        }

        $value = $default ?? $this->app->options[$option] ?? null;

        return $value;
    }

    # update option
    function update_option($option = '', $value = null)
    {
        update_option($this->prefix . $option, $value);
    }

    # delete option
    function delete_option($option = '')
    {
        delete_option($this->prefix . $option);
    }

    # add option
    function add_option($option = '', $value = null)
    {
        add_option($this->prefix . $option, $value);
    }

    # set transient
    function set_transient($key = '', $value = null, $expiration = 0)
    {
        set_transient($this->prefix . $key, $value, $expiration);
    }

    # get transient
    function get_transient($key = '', $default = null)
    {
        $value = get_transient($this->prefix . $key);

        if ($value !== null) {
            return $value;
        }

        return $default;
    }

    # delete transient
    function delete_transient($key = '')
    {
        delete_transient($this->prefix . $key);
    }


    # Remote data
    function remote_data()
    {
        $data = get_evr_remote_data();

        if($data && is_object($data) && isset($data->success) && $data->success){
            return $data->data;
        }

        return false;
    }
}
